package core.notevault.data;

import androidx.room.*;
import core.notevault.data.sync.SyncResponse;
import core.notevault.data.sync.SyncStatus;
import core.notevault.sync.synchronisation.songs.creation.SongCreationResponse;

import java.time.LocalDateTime;
import java.util.List;

@Dao
public interface MusicNoteDAO {

    @Insert
    long insert(MusicNote musicNote);

    @Insert
    void insertNoteSheet(NoteSheet noteSheet);

    @Query("SELECT * FROM music_notes")
    List<MusicNote> getAllNotes();

    @Query("SELECT * FROM note_sheets WHERE musicNoteId = :musicNoteId")
    List<NoteSheet> getNoteSheetsForMusicSong(long musicNoteId);

    @Query("SELECT * FROM concerts WHERE syncStatus = :status")
    List<Concert> getConcertsWithSyncStatus(SyncStatus status);

    @Query("SELECT * FROM music_notes WHERE syncStatus = :status")
    List<MusicNote> getSongsWithSyncStatus(SyncStatus status);

    @Insert
    void insertConcert(Concert concert);

    @Query("SELECT * FROM concerts")
    List<Concert> getAllConcerts();

    @Delete
    void deleteConcert(Concert concert);

    @Update
    void updateConcert(Concert concert);

    @Delete
    void deleteSong(MusicNote musicNote);

    @Insert
    void insertConcertSong(ConcertSong concertSong);

    @Query("SELECT m.* FROM music_notes m JOIN concert_songs cs ON m.musicNoteId = cs.musicNoteID WHERE cs.id = :concertID")
    List<MusicNote> getAllMusicNotesOfConcert(long concertID);

    @Query("DELETE FROM concert_songs WHERE concertID = :concertID AND musicNoteID = :songID")
    void deleteConcertSong(long songID, int concertID);

    @Transaction
    default void storeSongSyncResponses(List<SongCreationResponse> songCreationResponses) {
        for (SongCreationResponse response : songCreationResponses) {
            storeSongSyncResponse(response.getLocalID(), SyncStatus.SYNCED, response.getServerID());
        }
    }

    @Query("UPDATE music_notes SET syncStatus = :status, serverID = :serverID WHERE musicNoteId = :localID")
    void storeSongSyncResponse(long localID, SyncStatus status, String serverID);
}
