package core.notevault.sync.auth;

import android.content.Context;
import core.notevault.sync.APICallback;
import core.notevault.sync.ApiClient;
import core.notevault.sync.StatusResponse;
import core.notevault.sync.auth.apimodel.LoginRequest;
import core.notevault.sync.auth.apimodel.LoginResponse;
import core.notevault.sync.auth.apimodel.RegisterRequest;
import core.notevault.sync.auth.apimodel.RegisterResponse;
import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;

public class AuthRepository {
    private final AuthService authService;
    private final TokenManager tokenManager;

    public AuthRepository(Context context) {
        this.authService = ApiClient.getRetrofitInstance(context).create(AuthService.class);
        this.tokenManager = new TokenManager(context);
    }

    public void performLogin(String email, String password, APICallback callback) {
        LoginRequest loginRequest = new LoginRequest(email, password);

        authService.login(loginRequest).enqueue(new Callback<LoginResponse>() {
            @Override
            public void onResponse(Call<LoginResponse> call, Response<LoginResponse> response) {
                if (response.isSuccessful() && response.body() != null) {
                    String token = response.body().getToken();
                    saveToken(token);
                    // Erfolgsrückmeldung an den Callback senden
                    callback.onSuccess();
                } else {
                    // Fehlermeldung an den Callback senden
                    callback.onError("Login fehlgeschlagen. Überprüfe Benutzername und Passwort.");
                }
            }

            @Override
            public void onFailure(Call<LoginResponse> call, Throwable t) {
                // Netzwerkfehler an den Callback senden
                callback.onError("Netzwerkfehler: " + t.getMessage());
            }
        });
    }

    public void performRegistration(String email, String username, String password, APICallback callback) {
        RegisterRequest registerRequest = new RegisterRequest(username, password, email);
        authService.registration(registerRequest).enqueue(new Callback<StatusResponse>() {

            @Override
            public void onResponse(Call<StatusResponse> call, Response<StatusResponse> response) {
                if(response.isSuccessful() && response.body() != null) {
                    callback.onSuccess();
                } else {
                    callback.onError("Registration fehlgeschlagen. Überprüfe Benutzername und Passwort.");
                }
            }

            @Override
            public void onFailure(Call<StatusResponse> call, Throwable throwable) {
                callback.onError("Netzwerkfehler: " + throwable.getMessage());
            }
        });
    }

    private void saveToken(String token) {
        tokenManager.saveToken(token);
    }

    public String getToken() {
        return tokenManager.getToken();
    }

    public void logout() {
        tokenManager.clearToken();
    }
}
